#ifndef SUPER_STRING_HH
#define SUPER_STRING_HH

// Super String 1.0  3/29/99
// written by Phil Haubert and released into the public domain.

// string and buffer operations
// C compatibility
// extensible searching
// conversions and comparisons
// see external documentation

// original header comment, c.1993

// faster than a speeding parser, more powerful than a 4GL, able to munch
// a million characters in a single gulp:  it's ---

#define SS_STANDALONE

namespace std {
	template <class E> struct char_traits;
	template <class T> class allocator;
	template <class E, class T=char_traits<E>, class A=allocator<E> > class basic_string;
	template <class T, class A=allocator<T> > class vector;
	typedef basic_string<char> string;
	template <class E, class T=char_traits<E> > class basic_ostream;
	typedef basic_ostream <char, char_traits<char> > ostream;
}

// more predeclarations
class Urror;
class UrrorBadArg;
class UrrorBadState;
class UrrorNumberFormat;
class UrrorOutOfRange;
class UrrorOverflow;
class UrrorNotFound;
template <class T> struct Pair;


class SS {

  public:

	SS ();

	// allow an SS to be constructed for any type that supplies a non-member SSconvert
	template <class T>
	SS (T    const & t) { _zero(); SSconvert (t, *this); }
	SS (SS   const & s);
	SS (void const * v, int n);

	virtual ~SS ();

	// basic operations

	int  length () const;                // does not include null terminator
	int  size   () const;                // std::string has both size and length
	bool empty  () const;                // same as length()==0

	char       & operator [] (int ind);
	char const & operator [] (int ind) const;
	char       & at (int ind);
	char const & at (int ind) const;

	// allow as much compatibility with C strings as possible...
	operator char       * ();
	operator char const * () const;
	char       * c_str ();
	char const * c_str () const;
	char       * data  ();
	char const * data  () const;

	SS  clone () const;  // return a copy
	SS& erase ();        // set to ""

	// doubles as a buffer class -- these aren't string operations
	SS& buffer (int n);             // destroys contents of string
	// simple assign (to rep) - doesn't copy (or delete), or zero terminate
	SS& buffer (void const * v, int n);
	SS& resize (int n);             // preserves original contents
	SS& resizeToNullTerminator ();  // keep up to first embedded null

	// support a constructor to create a buffer
	class Buffer {
	  private:
		int _n;
		void const * _start;
		SS* _fillValue;
		bool _isAlloc;
		void copy (Buffer const & b);
	  public:
		// this creates a new string of length n, and zero terminated
		Buffer (int n=0, SS const & fillvalue="");
		Buffer (Buffer const & b);
		Buffer& operator = (Buffer const & b);
		// this assigns start directly to rep
		// a string created this way is not necessarily zero terminated
		Buffer (void const * start, int n);
		void const * getStart () const;
		int getSize () const;
		SS getFillValue () const;
		bool getAlloc () const;
		~Buffer ();
	};

	// some typedefs
	typedef std::string string;
	typedef Pair<int> BegLen;
	typedef std::vector< Pair<int> > BegLenVect;
	typedef std::vector<int> PosVect;
	// define these here and/or maybe in a separate "Umber" class
	typedef __int64 LongLong;
	typedef unsigned __int64 ULongLong;

  private:

	// allow use of SS in nested classes
	class SSCopyPointer {
	  private:
		SS* _rep;
	  public:
		SSCopyPointer (SS const & s);
		SSCopyPointer (SSCopyPointer const & p);
		SS       * operator -> ();
		SS const * operator -> () const;
		SS       & operator * ();
		SS const & operator * () const;
		operator = (SSCopyPointer const & p);
		~SSCopyPointer ();
	};

	class SubSS {
	  private:
		int _beg, _len;
		SS* _rep;
	  public:
		SubSS (SS* s, int beg, int len);
//		operator SS () const;
		SS get () const;
		SS& operator = (SS s);
	};

  public:

	// a substring is either a copy of a range or an object that can be assigned to
	// or decays, if copied, into a copy of the range
	SubSS sub (int beg, int len);
	SS    sub (int beg, int len) const;
	SubSS operator () (int beg, int len);
	SS    operator () (int beg, int len) const;
	SubSS operator () (Pair<int> const & beglen);
	SS    operator () (Pair<int> const & beglen) const;

	// bunch o' string operations

	// copy a range of chars into a new string
	SS  get     (int beg, int len=1)       const;
	SS  getFrom (int beg)                  const;
	SS  get     (Pair<int> const & beglen) const;
	// concatenates the ranges
	SS  get     (std::vector< Pair<int> > const & beglen) const;
	// copy the ranges into vector of SS
	std::vector<SS>& get (std::vector<SS>& s, std::vector< Pair<int> > const & beglen) const;

	// set does not extend string
	// the beg and len refers to the arg string
	SS& set (SS const & s, int pos=0);
	SS& set (SS const & s, int pos, int beg, int len);
	SS& set (SS const & s, int pos, Pair<int> const & beglen);
	// set s at each pos
	SS& set (SS const & s, std::vector<int> const & pos);
	// set an s at the corresponding pos
	SS& set (std::vector<SS> const & s, std::vector<int> const & pos);

	// removes some chars from string, returns the chars removed
	SS  cut     (int beg, int len=1);
	SS  cutFrom (int beg);
	SS  cut     (Pair<int> const & beglen);
	SS& cut     (SS& s, int beg, int len=1);      // returns arg/cut string
	SS& cut     (SS& s, Pair<int> const & beglen);
	// return s, containing all the cut strings
	std::vector<SS>& cut (std::vector<SS>& s, std::vector< Pair<int> > const & beglen);

	// insert the chars at pos. can extend string. pos==length is an append
	// the beg and len refers to the arg string
	SS& paste (SS const & s, int pos=0);
	SS& paste (SS const & s, int pos, int beg, int len);
	SS& paste (SS const & s, int pos, Pair<int> const & beglen);
	// paste s at each pos
	SS& paste (SS const & s, std::vector<int> const & pos);
	// paste an s at the corresponding pos
	SS& paste (std::vector<SS> const & s, std::vector<int> const & pos);

	// remove or replace a range of characters
	SS& removeRange (int beg=0, int len=fullength);
	SS& removeRange (Pair<int> const & beglen);
	SS& removeRange (std::vector< Pair<int> > const & beglen);
	SS& replaceRange (SS const & newseq, int beg=0, int len=fullength);
	SS& replaceRange (SS const & newseq, Pair<int> const & beglen);
	// replace each range with newseq
	SS& replaceRange (SS const & newseq, std::vector< Pair<int> > const & beglen);
	// replace each range with the corresponding s
	SS& replaceRange (std::vector<SS> const & s, std::vector< Pair<int> > const & beglen);

	// reverse the range of chars
	SS& reverse     (int beg=0, int len=fullength);
	SS& reverse     (Pair<int> const & beglen);
	SS& reverse     (std::vector< Pair<int> > const & beglen); // each range separate
	SS& itemReverse (std::vector< Pair<int> > const & beglen); // move around ranges
	SS& tailReverse (int len);

	// sort the chars in ascending order
	SS& sort     (int beg=0, int len=fullength);
	SS& sort     (Pair<int> const & beglen);
	SS& sort     (std::vector< Pair<int> > const & beglen); // each range separate
	SS& itemSort (std::vector< Pair<int> > const & beglen); // move around ranges
	SS& tailSort (int len);

	SS& fill (char c, int beg=0, int len=fullength);
	SS& fill (char c, Pair<int> const & beglen);
	SS& fill (char c, std::vector< Pair<int> > const & beglen);

	SS& repeat (SS const & s, int beg=0, int len=fullength);
	SS& repeat (SS const & s, Pair<int> const & beglen);
	SS& repeat (SS const & s, std::vector< Pair<int> > const & beglen);

	// fills with zeros, not set to "", more a buffer operation
	SS& zero ();
	SS& zero (int beg, int len);

	// get the chars at the beginning or end
	SS  head (int len) const;
	SS  tail (int len) const;

	char       & firstChar ();
	char const & firstChar () const;
	char       & lastChar  ();
	char const & lastChar  () const;

	// used in matching

	struct Finder;

	struct Find {
		Finder* _rep;
		Find ();
		Find (Find const & f);
		Find& operator = (Find const & f);
		virtual bool found (SS const & s, int pos, int& len) const;
		virtual Finder* clone () const;
		virtual ~Find ();
		// for stl - will only work for single char compares
		bool operator () (char c);
	};

	enum {notfound=-99999999,maxindex=-99999998,fullength=-99999997,allitems=-99999996};
	static const char nullchar;  // zero char
	static const Pair<int> nomatch; // if no match found

	// answer position or <0  ie notfound if fail
	int findNext (SS   const & sequence, int beg=0,        SS& result=nullref) const;
	int findPrev (SS   const & sequence, int beg=maxindex, SS& result=nullref) const;
	int findNext (Find const & finder,   int beg=0,        SS& result=nullref) const;
	int findPrev (Find const & finder,   int beg=maxindex, SS& result=nullref) const;
	int findNextNoCase (SS const & sequence, int beg=0,        SS& result=nullref) const;
	int findPrevNoCase (SS const & sequence, int beg=maxindex, SS& result=nullref) const;
	// find any char in set
	int findNextOf (SS const & charset, int beg=0,        SS& result=nullref) const;
	int findPrevOf (SS const & charset, int beg=maxindex, SS& result=nullref) const;

	// same as above but returns the beginning and length of match or SS::nomatch
	Pair<int> findNextMatch (SS   const & sequence, int beg=0)        const;
	Pair<int> findPrevMatch (SS   const & sequence, int beg=maxindex) const;
	Pair<int> findNextMatch (Find const & finder,   int beg=0)        const;
	Pair<int> findPrevMatch (Find const & finder,   int beg=maxindex) const;
	Pair<int> findNextNoCaseMatch (SS const & sequence, int beg=0)        const;
	Pair<int> findPrevNoCaseMatch (SS const & sequence, int beg=maxindex) const;

	// these throw ErrorNotFound
	SS findNextString (SS   const & sequence, int beg=0       ) const;
	SS findPrevString (SS   const & sequence, int beg=maxindex) const;
	SS findNextString (Find const & finder,   int beg=0       ) const;
	SS findPrevString (Find const & finder,   int beg=maxindex) const;

	SubSS findNextSubString (SS   const & sequence, int beg=0       );
	SubSS findPrevSubString (SS   const & sequence, int beg=maxindex);
	SubSS findNextSubString (Find const & finder,   int beg=0       );
	SubSS findPrevSubString (Find const & finder,   int beg=maxindex);

	// base find, end is the last index tried
	int find  (Find const & finder, int beg, int* len=0, int end=fullength, int inc=1) const;
	int rfind (Find const & finder, int beg, int* len=0, int end=fullength) const;
	int find  (SS const & sequence, int beg, int* len=0, int end=fullength, int inc=1) const;
	int rfind (SS const & sequence, int beg, int* len=0, int end=fullength) const;

	// does string contain the arg
	bool contains (SS   const & sequence, int beg=0, int len=fullength) const;
	bool contains (Find const & finder,   int beg=0, int len=fullength) const;
	bool contains (SS   const & sequence, Pair<int> const & beglen) const;
	bool contains (Find const & finder,   Pair<int> const & beglen) const;
	bool contains (SS   const & sequence, std::vector< Pair<int> > const & beglen) const;
	bool contains (Find const & finder,   std::vector< Pair<int> > const & beglen) const;

	// number of matches contained in string
	int population (SS   const & sequence, int beg=0, int len=fullength) const;
	int population (Find const & finder,   int beg=0, int len=fullength) const;
	int population (SS   const & sequence, Pair<int> const & beglen) const;
	int population (Find const & finder,   Pair<int> const & beglen) const;
	int population (SS   const & sequence, std::vector< Pair<int> > const & beglen) const;
	int population (Find const & finder,   std::vector< Pair<int> > const & beglen) const;

	// remove all occurrences in a region
	SS& remove (SS   const & oldseq, int beg=0, int len=fullength);
	SS& remove (Find const & finder, int beg=0, int len=fullength);
	SS& remove (SS   const & oldseq, Pair<int> const & beglen);
	SS& remove (Find const & finder, Pair<int> const & beglen);
	SS& remove (SS   const & oldseq, std::vector< Pair<int> > const & beglen);
	SS& remove (Find const & finder, std::vector< Pair<int> > const & beglen);

	// replace all occurrences in a region with newseq
	SS& replace (SS   const & oldseq, SS const & newseq, int beg=0, int len=fullength);
	SS& replace (Find const & finder, SS const & newseq, int beg=0, int len=fullength);
	SS& replace (SS   const & oldseq, SS const & newseq, Pair<int> const & beglen);
	SS& replace (Find const & finder, SS const & newseq, Pair<int> const & beglen);
	SS& replace (SS   const & oldseq, SS const & newseq, std::vector< Pair<int> > const & beglen);
	SS& replace (Find const & finder, SS const & newseq, std::vector< Pair<int> > const & beglen);

	// returns actual count of items removed or replaced
	// a value for count of allitems removes or replaces all matches
	int removeForward  (SS   const & oldseq, int count=1, int beg=0, int len=fullength);
	int removeForward  (Find const & finder, int count=1, int beg=0, int len=fullength);
	int removeBackward (SS   const & oldseq, int count=1, int beg=0, int len=fullength);
	int removeBackward (Find const & finder, int count=1, int beg=0, int len=fullength);

	int replaceForward  (SS   const & oldseq, SS const newseq, int count=1, int beg=0, int len=fullength);
	int replaceForward  (Find const & finder, SS const newseq, int count=1, int beg=0, int len=fullength);
	int replaceBackward (SS   const & oldseq, SS const newseq, int count=1, int beg=0, int len=fullength);
	int replaceBackward (Find const & finder, SS const newseq, int count=1, int beg=0, int len=fullength);

	// determines if chars starting at pos constitute a match
	bool match (SS   const & sequence, int pos, SS& result=nullref) const;
	bool match (Find const & finder,   int pos, SS& result=nullref) const;

	std::vector< Pair<int> >& match (SS   const & sequence, std::vector< Pair<int> >& beglen, int beg=0, int len=fullength) const;
	std::vector< Pair<int> >& match (Find const & finder,   std::vector< Pair<int> >& beglen, int beg=0, int len=fullength) const;
	std::vector< Pair<int> >& match (SS   const & sequence, std::vector< Pair<int> >& beglen, Pair<int> const & beglen_src) const;
	std::vector< Pair<int> >& match (Find const & finder,   std::vector< Pair<int> >& beglen, Pair<int> const & beglen_src) const;
	std::vector< Pair<int> >& match (SS   const & sequence, std::vector< Pair<int> >& beglen, std::vector< Pair<int> > const & beglen_src) const;
	std::vector< Pair<int> >& match (Find const & finder,   std::vector< Pair<int> >& beglen, std::vector< Pair<int> > const & beglen_src) const;

	// returns array of matching positions and lengths
	std::vector< Pair<int> >& matchForward  (SS   const & sequence, std::vector< Pair<int> >& beglen, int count=1, int beg=0, int len=fullength) const;
	std::vector< Pair<int> >& matchForward  (Find const & finder,   std::vector< Pair<int> >& beglen, int count=1, int beg=0, int len=fullength) const;
	std::vector< Pair<int> >& matchBackward (SS   const & sequence, std::vector< Pair<int> >& beglen, int count=1, int beg=0, int len=fullength) const;
	std::vector< Pair<int> >& matchBackward (Find const & finder,   std::vector< Pair<int> >& beglen, int count=1, int beg=0, int len=fullength) const;

	// returns array of start position and length of tokens, the inverse of match
	// uses whitespace, a string, or a Find predicate as delimiter
	std::vector< Pair<int> >& tokenize (std::vector< Pair<int> >& beglen) const;
	std::vector< Pair<int> >& tokenize (SS   const & delimiter, std::vector< Pair<int> >& beglen) const;
	std::vector< Pair<int> >& tokenize (Find const & finder,    std::vector< Pair<int> >& beglen) const;

	// a few predefined classes for the finding members
	// additional user predicates derive off Finder
	// this hierarchy assumes that objects will be copied as SS::Find and also
	// that objects are stored as SS::Find or as references/pointers to SS::Find

	struct Finder : public Find {
		Finder () : Find() { }
		Finder (Finder const & f) : Find() { } // intercept generated constructor
	};

	// all these should have a clone() and/or a copy constructor and destructor if needed
	struct FindChar : public Finder {
		char _c;
		FindChar (char c);
		bool found (SS const & s, int pos, int& len) const;
		Finder* clone () const;
	};
	struct FindCharNoCase : public FindChar {
		FindCharNoCase (char c);
		bool found (SS const & s, int pos, int& len) const;
		Finder* clone () const;
	};
	struct FindFunc : public Finder {
		typedef bool(*Func)(char c);
		Func _f;
		FindFunc (Func f);
		bool found (SS const & s, int pos, int& len) const;
		Finder* clone () const;
	};
	struct FindSet : public Finder {
		SSCopyPointer _set;
		FindSet (SS const & charset);
		bool found (SS const & s, int pos, int& len) const;
		Finder* clone () const;
	};
	struct FindString : public Finder {
		SSCopyPointer _str;
		FindString (SS const & str);
		bool found (SS const & s, int pos, int& len) const;
		Finder* clone () const;
	};
	struct FindVector : public Finder {
		typedef Find const & F;
		std::vector<Find const *>* _rep;
		FindVector (int n=0);
		FindVector (FindVector const & fv);
		void add (F f);
		virtual int size () const;
		virtual Find const & operator [] (int i) const;
		Finder* clone () const;
		~FindVector ();
	};
	struct FindSequence : public FindVector {
		FindSequence ();
		FindSequence (F f0, F f1);
		FindSequence (F f0, F f1, F f2);
		FindSequence (F f0, F f1, F f2, F f3);
		FindSequence (F f0, F f1, F f2, F f3, F f4);
		FindSequence (F f0, F f1, F f2, F f3, F f4, F f5);
		FindSequence (F f0, F f1, F f2, F f3, F f4, F f5, F f6);
		bool found (SS const & s, int pos, int& len) const;
		Finder* clone () const;
	};
	struct FindStringNoCase : public FindSequence {
		FindStringNoCase (SS const & s);
	};
	struct FindProxy : public Finder {
		Find const * _f;
		FindProxy (Find const & f);
		bool found (SS const & s, int pos, int& len) const;
		Finder* clone () const;
		~FindProxy ();
	  protected: FindProxy (FindProxy const & f);
	};
	struct FindNot : public FindProxy {
		FindNot (Find const & f);
		bool found (SS const & s, int pos, int& len) const;
		Finder* clone () const;
	  protected: FindNot (FindNot const & f);
	};
	struct FindOr : public FindVector {
		FindOr ();
		FindOr (F f0, F f1);
		FindOr (F f0, F f1, F f2);
		FindOr (F f0, F f1, F f2, F f3);
		FindOr (F f0, F f1, F f2, F f3, F f4);
		FindOr (F f0, F f1, F f2, F f3, F f4, F f5);
		FindOr (F f0, F f1, F f2, F f3, F f4, F f5, F f6);
		bool found (SS const & s, int pos, int& len) const;
		Finder* clone () const;
	};
	struct FindAnd : public FindVector {
		FindAnd ();
		FindAnd (F f0, F f1);
		FindAnd (F f0, F f1, F f2);
		FindAnd (F f0, F f1, F f2, F f3);
		FindAnd (F f0, F f1, F f2, F f3, F f4);
		FindAnd (F f0, F f1, F f2, F f3, F f4, F f5);
		FindAnd (F f0, F f1, F f2, F f3, F f4, F f5, F f6);
		bool found (SS const & s, int pos, int& len) const;
		Finder* clone () const;
	};
	struct FindCharCompare : public FindChar {
		typedef bool(*Comp)(char u, char w);
		Comp _comp;
		FindCharCompare (char w, Comp comp);
		bool found (SS const & s, int pos, int& len) const;
		Finder* clone () const;
	};
	struct FindCharGreaterThan : public FindCharCompare {
		FindCharGreaterThan (char w);
	};
	struct FindCharGreaterThanEqual : public FindCharCompare {
		FindCharGreaterThanEqual (char w);
	};
	struct FindCharLessThan : public FindCharCompare {
		FindCharLessThan (char w);
	};
	struct FindCharLessThanEqual : public FindCharCompare {
		FindCharLessThanEqual (char w);
	};
	struct FindCharInRange : public FindAnd {
		FindCharInRange (char w_low, char w_high);
	};
	struct FindCompare : public FindProxy {
		typedef bool(*Comp)(SS const & u, SS const & w);
		SSCopyPointer _w;
		Comp _comp;
		FindCompare (Find const & f, SS const & w, Comp comp);
		bool found (SS const & s, int pos, int& len) const;
		Finder* clone () const;
	};
	struct FindGreaterThan : public FindCompare {
		FindGreaterThan (Find const & f, SS const & w);
	};
	struct FindGreaterThanEqual : public FindCompare {
		FindGreaterThanEqual (Find const & f, SS const & w);
	};
	struct FindLessThan : public FindCompare {
		FindLessThan (Find const & f, SS const & w);
	};
	struct FindLessThanEqual : public FindCompare {
		FindLessThanEqual (Find const & f, SS const & w);
	};
	struct FindInRange : public FindAnd {
		FindInRange (Find const & f, SS const & w_low, SS const & w_high);
	};
	struct FindBool : public Finder {
		bool _b;
		int _l;
		FindBool (bool b, int l=0);
		bool found (SS const & s, int pos, int& len) const;
		Finder* clone () const;
	};
	struct FindPosition : public Finder {
		int _n;
		FindPosition (int n);
		bool found (SS const & s, int pos, int& len) const;
		Finder* clone () const;
	};
	struct FindRange : public Finder {
		int _n, _l;
		FindRange (int n, int l);
		bool found (SS const & s, int pos, int& len) const;
		Finder* clone () const;
	};
	struct FindDisplacedBy : public FindProxy {
		int _n;
		FindDisplacedBy (Find const & f, int n);
		bool found (SS const & s, int pos, int& len) const;
		Finder* clone () const;
	};
	struct FindMultiple : public FindProxy {
		int _n,_m;
		FindMultiple (Find const & f, int n=1);
		FindMultiple (Find const & f, int n_min, int n_max);
		bool found (SS const & s, int pos, int& len) const;
		Finder* clone () const;
	  protected: FindMultiple (FindMultiple const & f);
	};
	struct FindZeroOrMore : public FindMultiple {
		FindZeroOrMore (Find const & f);
	};
	struct FindOneOrMore : public FindMultiple {
		FindOneOrMore (Find const & f);
	};
	struct FindZeroOrOne : public FindMultiple {
		FindZeroOrOne (Find const & f);
	};
	struct FindAtLeast : public FindMultiple {
		FindAtLeast (Find const & f, int n);
	};
	struct FindUpTo : public FindMultiple {
		FindUpTo (Find const & f, int n);
	};
	struct FindAfter : public FindVector {
		FindAfter (F f0, F f1);
		bool found (SS const & s, int pos, int& len) const;
		Finder* clone () const;
	};
	struct FindWithin : public FindAfter {
		FindWithin (F f0, F f1);
		bool found (SS const & s, int pos, int& len) const;
		Finder* clone () const;
	};
	struct FindSeparatedBy : public FindAfter {
		int _n;
		FindSeparatedBy (F f0, F f1, int n);
		bool found (SS const & s, int pos, int& len) const;
		Finder* clone () const;
	};
	struct FindCloserThan : public FindSeparatedBy {
		FindCloserThan (F f0, F f1, int n);
		bool found (SS const & s, int pos, int& len) const;
		Finder* clone () const;
	};
	struct FindFartherThan : public FindSeparatedBy {
		FindFartherThan (F f0, F f1, int n);
		bool found (SS const & s, int pos, int& len) const;
		Finder* clone () const;
	};
	struct FindDelimit : public Finder {
		char _c0,_c1;
		FindDelimit (char c);
		FindDelimit (char left, char right);
		bool found (SS const & s, int pos, int& len) const;
		Finder* clone () const;
	};

	// a few predefined objects for the finding members

	static const FindFunc whitespace;
	static const FindFunc blackspace;
	static const FindFunc lowercase;
	static const FindFunc uppercase;
	static const FindFunc alpha;
	static const FindFunc digit;
	static const FindFunc alphanumeric;
	static const FindFunc punct;
	static const FindFunc printable;
	static const FindFunc hexdigit;
	static const FindFunc cntrl;
	static const FindFunc graph;
	static const FindBool findtrue;
	static const FindBool findfalse;
	static const FindBool anychar;
	static const FindPosition frontposition;
	static const FindPosition backposition;
	static const FindOr endofline;  // the position of the char before eol char(s)
	static const FindDelimit singlequotedelimit;
	static const FindDelimit doublequotedelimit;
	static const FindDelimit parendelimit;

	// a region or ref is simply a char* to an internal part of the parent string
	// it will not in general be null-terminated
	// it will be invalid when the (ultimate) parent goes out of scope
	// if a ref is copied, the new string will also be a ref
	SS getRegion (int beg=0, int len=fullength) const;
	SS getRegion (Pair<int> const & beglen) const;
	std::vector<SS>& getRegion (std::vector<SS>& s, std::vector< Pair<int> > const & beglen) const;

	// stl iterators
	char       * begin ();
	char const * begin () const;
	char       * end   ();
	char const * end   () const;

	// copying
	template <class T>
	SS& operator = (T  const & t) { SSconvert (t,*this); return *this; }
	SS& operator = (SS const & s) { assign (s); return *this; }

	// member concatenation
	template <class T>
	SS operator + (T const & t) const { return concatenate (*this,t); }
	template <class T>
	SS& operator += (T const & t) { assign (concatenate (*this, t)); return *this; }

	// member relational operators
	template <class T>
	bool operator == (T const & t) const { return SScompare (*this, t) == 0; }
	template <class T>
	bool operator != (T const & t) const { return SScompare (*this, t) != 0; }
	template <class T>
	bool operator <  (T const & t) const { return SScompare (*this, t) <  0; }
	template <class T>
	bool operator >  (T const & t) const { return SScompare (*this, t) >  0; }
	template <class T>
	bool operator <= (T const & t) const { return SScompare (*this, t) <= 0; }
	template <class T>
	bool operator >= (T const & t) const { return SScompare (*this, t) >= 0; }

	// resolve some ambiguity
	SS operator + (SS s);
	SS operator + (SS s) const;
	bool operator == (SS s);
	bool operator == (SS s) const;
	bool operator != (SS s);
	bool operator != (SS s) const;
	bool operator <  (SS s);
	bool operator <  (SS s) const;
	bool operator >  (SS s);
	bool operator >  (SS s) const;
	bool operator <= (SS s);
	bool operator <= (SS s) const;
	bool operator >= (SS s);
	bool operator >= (SS s) const;

	// these construct char representations of the numbers
	// this is what is means to convert a number to a string
	void assign (SS            const & s);
	void assign (string        const & s);
	void assign (char          const * s);
	void assign (unsigned char const * s);
	void assign (signed char   const * s);
	void assign (char           v);
	void assign (unsigned char  v);
	void assign (signed char    v);
	void assign (bool           v);
	void assign (short          v);
	void assign (unsigned short v);
	void assign (int            v);
	void assign (unsigned int   v);
	void assign (long           v);
	void assign (unsigned long  v);
	void assign (LongLong       v);
	void assign (ULongLong      v);
	void assign (float          v);
	void assign (double         v);
	void assign (long double    v);
	void assign (std::vector<char> const & v);
	void assign (Buffer const & v);
	void assign (void const * v, int n);

	int compare (SS            const & s) const;
	int compare (string        const & s) const;
	int compare (char          const * s) const;
	int compare (unsigned char const * s) const;
	int compare (signed char   const * s) const;
	int compare (char           v) const;
	int compare (unsigned char  v) const;
	int compare (signed char    v) const;
	int compare (bool           v) const;
	int compare (short          v) const;
	int compare (unsigned short v) const;
	int compare (int            v) const;
	int compare (unsigned int   v) const;
	int compare (long           v) const;
	int compare (unsigned long  v) const;
	int compare (LongLong       v) const;
	int compare (ULongLong      v) const;
	int compare (float          v) const;
	int compare (double         v) const;
	int compare (long double    v) const;
	int compare (std::vector<char> const & s) const;

	int  compareNoCase (SS   const & s) const;
	int  compare       (void const * v, int n) const;
	bool compare       (Find const & f) const; // does f match the whole string?

	bool isUpperCase    (int pos) const;
	bool isLowerCase    (int pos) const;
	bool isWhiteSpace   (int pos) const;
	bool isBlackSpace   (int pos) const;
	bool isAlpha        (int pos) const;
	bool isDigit        (int pos) const;
	bool isAlphaNumeric (int pos) const;
	bool isPunct        (int pos) const;
	bool isPrintable    (int pos) const;
	bool isHexDigit     (int pos) const;
	bool isCntrl        (int pos) const;
	bool isGraph        (int pos) const;

	SS& toLower ();
	SS& toLower (int pos);
	SS& toUpper ();
	SS& toUpper (int pos);

	SS toHex     () const;
	SS fromHex   () const;
	SS outputHex () const;  // pretty print in hex

	SS trim () const;
	SS dup (int n=1) const; // new string of n copies

	char* extract ();       // take over memory management of rep

	long hash () const;

	SS& become (SS& s);     // arg is left as ""
	SS& swap   (SS& s);
	SS& swap   (int pos0, int pos1);

	static void copy     (void       * dst, void const * src, int n);
	static void copy     (void       * dst, int l0, void const * src, int l1);
	static int  compare  (void const * bf0, void const * bf1, int n);
	static int  compare  (void const * bf0, int l0, void const * bf1, int l1);
	static void fill     (void       * dst, char c, int n);
	static int  findNext (void const * dat, char c, int n);

	// zero out arbitrary object
	template <class T>
	static inline void zero       (T& t) { SS::fill (&t, nullchar, sizeof(T)); }
	template <class T>
	static inline void fillObject (T& t, char c) { SS::fill (&t, c, sizeof(T)); }
	template <class T>
	static inline SS   fromObject (T& t) { SS ss; return ss.assignFromObject (t); }

	// beg refers to string object
	SS const & copyTo     (void* dst, int n=fullength, int beg=0) const;
	SS       & copyFrom   (void* src, int n=fullength, int beg=0);  // won't extend string
	SS       & assignFrom (void* src, int n);                       // reassigns string
	template <class T>
	SS const & copyToObject     (T& t, int beg=0) const {return copyTo    (&t, sizeof(T), beg);}
	template <class T>
	SS       & copyFromObject   (T& t, int beg=0)       {return copyFrom  (&t, sizeof(T), beg);}
	template <class T>
	SS       & assignFromObject (T& t)                  {return assignFrom(&t, sizeof(T));     }

	static SS concatenate (SS const & u, SS const & w);
	static SS concatenate (std::vector<SS>& s);

	static long hash (void const * v, int n);
	static int  strLen (char const * s);
	static int  strLen (signed char const * s);
	static int  strLen (unsigned char const * s);

	static bool upperP     (char c);
	static bool lowerP     (char c);
	static bool whiteP     (char c);
	static bool blackP     (char c);
	static bool alphaP     (char c);
	static bool digitP     (char c);
	static bool alphanumP  (char c);
	static bool punctP     (char c);
	static bool printableP (char c);
	static bool hexDigitP  (char c);
	static bool cntrlP     (char c);
	static bool graphP     (char c);

	static void to_lower (char& c);
	static void to_upper (char& c);
	static char getLower (char c);
	static char getUpper (char c);

	// allow conversions to be added by user and explicitly invoked
	template <class T>
	inline static SS toSS (T const & t) { SS s; SSconvert (t,s); return s; }

	template <class T>
	SS const & toType (T & t) const { SSconvertFrom (*this,t); return *this; }

	// number conversions
	bool           toBool      ()         const;
	short          toShort     ()         const;
	unsigned short toUShort    ()         const;
	int            toInt       ()         const;
	unsigned int   toUInt      ()         const;
	long           toLong      ()         const;
	unsigned long  toULong     ()         const;
	LongLong       toLongLong  ()         const;
	ULongLong      toULongLong ()         const;
	short          toShort     (int base) const;
	unsigned short toUShort    (int base) const;
	int            toInt       (int base) const;
	unsigned int   toUInt      (int base) const;
	long           toLong      (int base) const;
	unsigned long  toULong     (int base) const;
	LongLong       toLongLong  (int base) const;
	ULongLong      toULongLong (int base) const;
	float          toFloat     ()         const;
	double         toDouble    ()         const;
	long double    toLongDouble()         const;

	string         toString    ()         const;
	std::vector<char> toVector ()         const;

	void toInteger (short          & number) const;
	void toInteger (unsigned short & number) const;
	void toInteger (int            & number) const;
	void toInteger (unsigned int   & number) const;
	void toInteger (long           & number) const;
	void toInteger (unsigned long  & number) const;
	void toInteger (LongLong       & number) const;
	void toInteger (ULongLong      & number) const;
	void toInteger (short          & number, int base) const;
	void toInteger (unsigned short & number, int base) const;
	void toInteger (int            & number, int base) const;
	void toInteger (unsigned int   & number, int base) const;
	void toInteger (long           & number, int base) const;
	void toInteger (unsigned long  & number, int base) const;
	void toInteger (LongLong       & number, int base) const;
	void toInteger (ULongLong      & number, int base) const;

	static SS toBase (short          number, int base=10);
	static SS toBase (unsigned short number, int base=10);
	static SS toBase (int            number, int base=10);
	static SS toBase (unsigned int   number, int base=10);
	static SS toBase (long           number, int base=10);
	static SS toBase (unsigned long  number, int base=10);
	static SS toBase (LongLong       number, int base=10);
	static SS toBase (ULongLong      number, int base=10);

	static SS sprint (const char * fmt, ...);

	// exceptions that can be thrown
#ifndef SS_STANDALONE
	typedef Urror Error;
	typedef UrrorBadArg ErrorBadArg;
	typedef UrrorBadState ErrorBadState;
	typedef UrrorNumberFormat ErrorNumberFormat;
	typedef UrrorOutOfRange ErrorOutOfRange;
	typedef UrrorOverflow ErrorOverflow;
	typedef UrrorNotFound ErrorNotFound;
#else
	class Error {
	  public:
		Error (SS const & s) : _s(s) { }
		char const * what() const { return *_s; }
	  private:
		SSCopyPointer _s;
	};
	typedef Error ErrorBadArg;
	typedef Error ErrorBadState;
	typedef Error ErrorNumberFormat;
	typedef Error ErrorOutOfRange;
	typedef Error ErrorOverflow;
	typedef Error ErrorNotFound;
#endif

	// see if args are positive and in range
	bool verify (int ind) const;
	bool verify (int beg, int len) const;
	// these aren't part of the api but are needed in the impl
	void adjust (int& beg, int& len) const;  // adjusts beg and len to fit
	void adjust (Pair<int>& beglen)  const;
	static bool alreadySorted (std::vector<int> const & v);
	static bool alreadySorted (std::vector< Pair<int> > const & v);
	static void sortBegLenVect (std::vector<int> const & v, std::vector<int> & o);
	static void sortBegLenVect (std::vector< Pair<int> > const & v, std::vector<int> & o);

  protected:

    int   _len;
    char* _rep;
	int   _flags;
	int   _allocLength;

	enum Flags {Alloc=1, Ref=2};
	static SS& nullref;
	static char const * formatStringLongLong;
	static char const * formatStringULongLong;

	int translateIndex (int ind) const;
	int translateLength (int len) const;
	int translateCheck (int ind) const;  // produces an out-of-range error
	void check  (int  ind)           const;  // produces an out-of-range error
	void adjust (int& ind)           const;  // adjusts ind to be inside string
	bool findCheck (int beg, int end, int inc) const;
	void lenAdjust (int& len) const;         // check for invalid length
	void pasteAdjust (int& ind) const;         // paste index can be length
	void modAdjust (int& beg, int& end, int inc) const;
	bool matchAdjust (int& beg) const;
	bool verify3 (SS const & s, int pos, int beg, int len) const;
	bool adjust3 (SS const & s, int& p, int& b, int& l) const;
	int _tryInd (int i) const;

	bool isAlloc    () const;
	void setAlloc   ();
	void setNoAlloc ();
	bool isRef      () const;
	void setRef     ();
	void setNoRef   ();
	void checkFlags () const;

	bool addressInside (void const * p) const;

	enum { HeadSample=20 };

	// give subclass a way in
	class DoNothing { };
    SS (DoNothing);

    virtual void destroy();

	void  _leng  (int n);
	void  _data  (void const * s);
	void  _data  (string const & s);
	void  _data  (std::vector<char> const & v);
	void  _setData (char * rep, int len, int flags, int allocLength);
	void  _setData (SS const & s);
	void  _zero  ();
	void  _null  ();

	// maybe put this into a separate "Umber" class
	template <class T> static inline int compareNumeric (T a, T b) {
		if (a>b) return 1;
		if (a<b) return -1;
		return 0;
	}
	static int compareBool (bool a, bool b);
	static int compareString (char const * a, int l0, string const & b);
	static int compareVector (char const * a, int l0, std::vector<char> const & b);
	int compareChar (char b) const;

	int findChar (char c, int beg, int end, int inc) const;
	int findFinder (Find const & finder, int beg, int end, int inc, int* len) const;
	int findString (SS const & s, int beg, int end, int inc) const;
	int findStringHelper (SS const & s, int beg, int end, int inc) const;
	bool matchString (char const * s, int len, int pos) const;
	int doFind (Find const & finder, int beg, SS& result, int inc) const;
	int doFindString (SS const & s, int beg, SS& result, int inc) const;
	Pair<int> doFindMatch (Find const & finder, int beg, int inc) const;
	Pair<int> doFindStringMatch (SS const & s, int beg, int inc) const;

	SS makeHeadSample () const;

	friend class SubSS;
	friend struct FindString;
	friend void SSconvert (SubSS const & u, SS & w) { w = u.get(); }

};

#ifdef SS_STANDALONE
template <class T>
struct Pair { T _0; T _1; Pair(T t0,T t1) : _0(t0),_1(t1) {} Pair () {} };
#endif

// non-member conversion functions
void SSconvert (SS const & u,                SS & w);
void SSconvert (char const * u,              SS & w);
void SSconvert (unsigned char const * u,     SS & w);
void SSconvert (signed char const * u,       SS & w);
void SSconvert (char u,                      SS & w);
void SSconvert (unsigned char u,             SS & w);
void SSconvert (signed char u,               SS & w);
void SSconvert (bool u,                      SS & w);
void SSconvert (short u,                     SS & w);
void SSconvert (unsigned short u,            SS & w);
void SSconvert (int u,                       SS & w);
void SSconvert (unsigned int u,              SS & w);
void SSconvert (long u,                      SS & w);
void SSconvert (unsigned long u,             SS & w);
void SSconvert (SS::LongLong u,              SS & w);
void SSconvert (SS::ULongLong u,             SS & w);
void SSconvert (float u,                     SS & w);
void SSconvert (double u,                    SS & w);
void SSconvert (long double u,               SS & w);
void SSconvert (SS::string        const & u, SS & w);
void SSconvert (std::vector<char> const & u, SS & w);
void SSconvert (SS::Buffer        const & u, SS & w);

void SSconvertFrom (SS const & u, SS                    & w);
void SSconvertFrom (SS const & u, char const          * & w);
void SSconvertFrom (SS const & u, unsigned char const * & w);
void SSconvertFrom (SS const & u, signed char const   * & w);
void SSconvertFrom (SS const & u, char                  & w);
void SSconvertFrom (SS const & u, unsigned char         & w);
void SSconvertFrom (SS const & u, signed char           & w);
void SSconvertFrom (SS const & u, bool                  & w);
void SSconvertFrom (SS const & u, short                 & w);
void SSconvertFrom (SS const & u, unsigned short        & w);
void SSconvertFrom (SS const & u, int                   & w);
void SSconvertFrom (SS const & u, unsigned int          & w);
void SSconvertFrom (SS const & u, long                  & w);
void SSconvertFrom (SS const & u, unsigned long         & w);
void SSconvertFrom (SS const & u, SS::LongLong          & w);
void SSconvertFrom (SS const & u, SS::ULongLong         & w);
void SSconvertFrom (SS const & u, float                 & w);
void SSconvertFrom (SS const & u, double                & w);
void SSconvertFrom (SS const & u, long double           & w);
void SSconvertFrom (SS const & u, SS::string            & w);
void SSconvertFrom (SS const & u, std::vector<char>     & w);
void SSconvertFrom (SS const & u, SS::Buffer            & w);

// non-member comparison functions
int SScompare (SS const & u, SS                const & w);
int SScompare (SS const & u, char              const * w);
int SScompare (SS const & u, unsigned char     const * w);
int SScompare (SS const & u, signed char       const * w);
int SScompare (SS const & u, char                      w);
int SScompare (SS const & u, unsigned char             w);
int SScompare (SS const & u, signed char               w);
int SScompare (SS const & u, bool                      w);
int SScompare (SS const & u, short                     w);
int SScompare (SS const & u, unsigned short            w);
int SScompare (SS const & u, int                       w);
int SScompare (SS const & u, unsigned int              w);
int SScompare (SS const & u, long                      w);
int SScompare (SS const & u, unsigned long             w);
int SScompare (SS const & u, SS::LongLong              w);
int SScompare (SS const & u, SS::ULongLong             w);
int SScompare (SS const & u, float                     w);
int SScompare (SS const & u, double                    w);
int SScompare (SS const & u, long double               w);
int SScompare (SS const & u, SS::string        const & w);
int SScompare (SS const & u, std::vector<char> const & w);

// non-member concatenation
template <class T>
inline SS operator + (T const & u, SS const & w) { return SS::concatenate (u,w); }

// non-member relational operators
template <class T>
inline bool operator == (T const & u, SS const & w) { return SScompare (w,u) == 0; }
template <class T>
inline bool operator != (T const & u, SS const & w) { return SScompare (w,u) != 0; }
template <class T>
inline bool operator <  (T const & u, SS const & w) { return SScompare (w,u) >  0; }
template <class T>
inline bool operator >  (T const & u, SS const & w) { return SScompare (w,u) <  0; }
template <class T>
inline bool operator <= (T const & u, SS const & w) { return SScompare (w,u) >= 0; }
template <class T>
inline bool operator >= (T const & u, SS const & w) { return SScompare (w,u) <= 0; }


// insert string into an ostream
std::ostream& operator << (std::ostream& os, SS const & s);

// inlines
inline int SS::length () const { return _len; }
inline int SS::size () const { return _len; }
inline bool SS::empty  () const { return _len==0; }
inline SS::operator char* () { return _rep; }
inline SS::operator char const * () const { return _rep; }
inline char* SS::c_str () { return _rep; }
inline char const * SS::c_str () const { return _rep; }
inline char* SS::data () { return _rep; }
inline char const * SS::data () const { return _rep; }
inline char * SS::begin () { return _rep; }
inline char const * SS::begin () const { return _rep; }
inline char * SS::end () { return _rep + _len; }
inline char const * SS::end () const { return _rep + _len; }
inline int SS::_tryInd (int i) const { return i>=0 && i<_len ? i : translateCheck(i); }
inline char & SS::operator [] (int ind) { return _rep[_tryInd(ind)]; }
inline char const & SS::operator [] (int ind) const { return _rep[_tryInd(ind)]; }
inline char & SS::at (int ind) { return _rep[_tryInd(ind)]; }
inline char const & SS::at (int ind) const { return _rep[_tryInd(ind)]; }

// end SUPER_STRING_HH
#endif
